
// MDBSlaveReaderDlg.cpp : implementation file
//--------------------------------------------
#include "stdafx.h"
#include "MDBSlaveReader.h"
#include "MDBSlaveReaderDlg.h"
#include "SettingDlg.h"
#include "MDBdefines.h"
#include "InterfaceSocket.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// CMDBSlaveReaderDlg dialog
//--------------------------

CMDBSlaveReaderDlg::CMDBSlaveReaderDlg(CWnd* pParent) : CDialog(CMDBSlaveReaderDlg::IDD, pParent)
//===============================================================================================
{
	//{{AFX_DATA_INIT(CMDBSlaveReaderDlg)
	m_strDisplay = _T("");
	m_bEnabled = FALSE;
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon=AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	m_bJustReset=true;
	m_bVendApproved=false;
	m_bVendSuccess=false;
	m_bVendFailure=false;
	m_bSessionComplete=false;
	m_dwCredit=0;
	m_dwPrice=0;
	m_dwItem=0;
}

void CMDBSlaveReaderDlg::DoDataExchange(CDataExchange* pDX)
//=========================================================
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMDBSlaveReaderDlg)
	DDX_Text(pDX, IDC_DISPLAY, m_strDisplay);
	DDX_Check(pDX, IDC_ENABLED, m_bEnabled);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CMDBSlaveReaderDlg, CDialog)
	//{{AFX_MSG_MAP(CMDBSlaveReaderDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_COMMAND(IDM_SETTINGS, OnSettings)
	ON_BN_CLICKED(IDC_RESET, OnReset)
	ON_BN_CLICKED(IDC_CARD_INSERT, OnCardInsert)
	ON_BN_CLICKED(IDC_CARD_REMOVE, OnCardRemove)
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// CMDBSlaveReaderDlg message handlers
//------------------------------------

BOOL CMDBSlaveReaderDlg::OnInitDialog()
//=====================================
{
	BYTE pData[16];
	CMDBSlaveReaderApp* pApp=(CMDBSlaveReaderApp*)AfxGetApp();	
	CDialog::OnInitDialog();

	// Set the icon for this dialog.
	// The framework does this automatically when
	// the application's main window is not a dialog
	//----------------------------------------------
	SetIcon(m_hIcon,TRUE); // Set big icon
	SetIcon(m_hIcon,FALSE); // Set small icon

	// Enable MDB card reader
	//-----------------------
	if (pApp->IsConnected())
	{
		pData[0]=0x07; // Option type
		pData[1]=0x03; // Option length
		pData[2]=0x12; // Option data, (Card reader response)
		pApp->GetControlSocket()->Send(pData,3);
	}
	return TRUE; // Return TRUE  unless you set the focus to a control
}

void CMDBSlaveReaderDlg::OnDestroy() 
//==================================
{
	BYTE pData[16];
	CMDBSlaveReaderApp* pApp=(CMDBSlaveReaderApp*)AfxGetApp();	
	CDialog::OnDestroy();
	
	// Disable MDB card reader
	//------------------------
	if (pApp->IsConnected())
	{
		pData[0]=0x07; // Option type
		pData[1]=0x03; // Option length
		pData[2]=0x00; // Option data, (Card reader response)
		pApp->GetControlSocket()->Send(pData,3);
	}
}

// If you add a minimize button to your dialog,
// you will need the code below to draw the icon.
// For MFC applications using the document/view model,
// this is automatically done for you by the framework.
//-----------------------------------------------------

void CMDBSlaveReaderDlg::OnPaint() 
//================================
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting
		SendMessage(WM_ICONERASEBKGND,(WPARAM)dc.GetSafeHdc(),0);

		// Center icon in client rectangle
		//--------------------------------
		int cxIcon=GetSystemMetrics(SM_CXICON);
		int cyIcon=GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x=(rect.Width()-cxIcon+1)/2;
		int y=(rect.Height()-cyIcon+1)/2;

		// Draw the icon
		//--------------
		dc.DrawIcon(x,y,m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display
// while the user drags the minimized window.
//------------------------------------------------------

HCURSOR CMDBSlaveReaderDlg::OnQueryDragIcon()
//-------------------------------------------
{
	return (HCURSOR)m_hIcon;
}

void CMDBSlaveReaderDlg::OnSettings() 
//===================================
{
	CSettingDlg SettingDlg;
	SettingDlg.DoModal();
}

void CMDBSlaveReaderDlg::Display(DWORD dwAmount)
//==============================================
{
	m_strDisplay.Format(_T("Credit: %u.%02u"),dwAmount/100,dwAmount%100);
	UpdateData(false);
}

void CMDBSlaveReaderDlg::Received(BYTE* pData, DWORD dwLength)
//============================================================
{
	BYTE pFrame[MAX_FRAME_LENGTH];
	CMDBSlaveReaderApp* pApp=(CMDBSlaveReaderApp*)AfxGetApp();	
	switch (pData[0]&0x07)
	{
		case CMD_RESET:
		{
			break;
		}
		case CMD_SETUP:
		{
			if (pData[1]==SUB_CONFIG_DATA)
			{
				pFrame[0]=ADR_READER;
				pFrame[1]=RSP_READER_CONFIG_DATA;
				pFrame[2]=0x02; // Reader future level 2
				pFrame[3]=0x19; // Country code high 
				pFrame[4]=0x78; // Country code low
				pFrame[5]=0x01; // Scale factor 1
				pFrame[6]=0x02; // Decimal places 2
				pFrame[7]=0x05; // Application maximum response time
				pFrame[8]=0x0F; // Options
				pApp->GetCommunicationSocket()->Send(pFrame,9);
			}
			break;
		}
		case CMD_POLL:
		{
			if (m_bJustReset)
			{
				m_bJustReset=false;
				pFrame[0]=ADR_READER;
				pFrame[1]=RSP_JUST_RESET;
				pApp->GetCommunicationSocket()->Send(pFrame,2);
				Display(0);
				m_dwCredit=1000;
				break;
			}
			if (m_bVendApproved)
			{
				m_bVendApproved=false;
				pFrame[0]=ADR_READER;
				pFrame[1]=RSP_VEND_APPROVED;
				pFrame[2]=(BYTE)(m_dwPrice>>8);
				pFrame[3]=(BYTE)m_dwPrice;
				pApp->GetCommunicationSocket()->Send(pFrame,4);
				break;
			}
			if (m_bVendSuccess)
			{
				m_bVendSuccess=false;
				pFrame[0]=ADR_READER;
				pFrame[1]=RSP_SESSION_CANCEL_REQUEST;
				pApp->GetCommunicationSocket()->Send(pFrame,2);
				break;
			}
			if (m_bVendFailure)
			{
				m_bVendFailure=false;
				pFrame[0]=ADR_READER;
				pFrame[1]=RSP_SESSION_CANCEL_REQUEST;
				pApp->GetCommunicationSocket()->Send(pFrame,2);
				break;
			}
			if (m_bSessionComplete)
			{
				m_bSessionComplete=false;
				pFrame[0]=ADR_READER;
				pFrame[1]=RSP_END_SESSION;
				pApp->GetCommunicationSocket()->Send(pFrame,2);
				break;
			}
			break;
		}
		case CMD_VEND:
		{
			if (pData[1]==SUB_VEND_REQUEST)
			{
				m_dwPrice=(pData[2]<<8)+pData[3];
				m_dwItem=(pData[4]<<8)+pData[5];
				m_dwCredit-=m_dwPrice;
				Display(m_dwCredit);
				m_bVendApproved=true;
				break;
			}
			if (pData[1]==SUB_VEND_SUCCESS)
			{
				m_bVendSuccess=true;
				break;
			}
			if (pData[1]==RSP_VEND_DENIED)
			{
				m_bVendFailure=true;
				m_dwCredit+=m_dwPrice;
				Display(m_dwCredit);
				break;
			}
			if (pData[1]==SUB_VEND_SESSION_COMPLETE)
			{
				m_bSessionComplete=true;
				break;
			}
			break;
		}
		case CMD_READER:
		{
			if (pData[1]==SUB_READER_DISABLE)
			{
				m_bEnabled=false;
				UpdateData (false);
				break;
			}
			if (pData[1]==SUB_READER_ENABLE)
			{
				m_bEnabled=true;
				UpdateData (false);
				break;
			}
			break;
		}
		case CMD_REVALUE:
		{
			break;
		}
		case CMD_EXPANSION:
		{
			if (pData[1]==SUB_REQUEST_ID)
			{
				pFrame[0]=ADR_READER;
				pFrame[1]=RSP_PERIPHERAL_ID;
				strcpy((char*)&pFrame[2],_T("BDS"));
				strcpy((char*)&pFrame[5],_T("BD-4000-U"));
				strcpy((char*)&pFrame[17],_T("MDB SLAVE"));
				pFrame[29]=0x01; // Version high
				pFrame[30]=0x00; // Version low
				pApp->GetCommunicationSocket()->Send(pFrame,31);
			}
			break;
		}
	}
}

void CMDBSlaveReaderDlg::OnCardInsert() 
//=====================================
{
	BYTE pFrame[MAX_FRAME_LENGTH];
	CMDBSlaveReaderApp* pApp=(CMDBSlaveReaderApp*)AfxGetApp();	
	pFrame[0]=ADR_READER;
	pFrame[1]=RSP_BEGIN_SESSION;
	pFrame[2]=(BYTE)(m_dwCredit>>8);
	pFrame[3]=(BYTE)m_dwCredit;
	pFrame[4]=0xFF; // Unknown media ID
	pFrame[5]=0xFF;
	pFrame[6]=0xFF;
	pFrame[7]=0xFF;
	pFrame[8]=0x00; // Type of payment
	pFrame[9]=0x00; // Payment data
	pFrame[10]=0x00;
	pApp->GetCommunicationSocket()->Send(pFrame,11);
	Display(m_dwCredit);
}

void CMDBSlaveReaderDlg::OnCardRemove() 
//=====================================
{
	BYTE pFrame[MAX_FRAME_LENGTH];
	CMDBSlaveReaderApp* pApp=(CMDBSlaveReaderApp*)AfxGetApp();	
	pFrame[0]=ADR_READER;
	pFrame[1]=RSP_SESSION_CANCEL_REQUEST;
	pApp->GetCommunicationSocket()->Send(pFrame,2);
	Display(0);
}

void CMDBSlaveReaderDlg::OnReset() 
//================================
{
	BYTE pFrame[MAX_FRAME_LENGTH];
	CMDBSlaveReaderApp* pApp=(CMDBSlaveReaderApp*)AfxGetApp();	
	pFrame[0]=ADR_READER;
	pFrame[1]=RSP_JUST_RESET;
	pApp->GetCommunicationSocket()->Send(pFrame,2);
}
